// jkBindTest.cpp : Defines the entry point for the console application.
//

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <stdio.h>
#include <tchar.h>
#include <iostream>
#include <string>
#include <string.h>

#include <script/VMCore.h>

using namespace std;

enum MyEnum { val1, val2 };

class Basic
{
public:
    virtual void print(const xchar* string)
    {
        wcout << "basic: " << string << endl;
    }

    static int get10() 
    {
        wcout << "static call" << endl;
        return 10;
    }

    void ec(MyEnum cc)
    {
    }
};

namespace foo {

class Super: public Basic
{
public:
    Super(const xchar* nn)
    {
        _myName = nn;
    }

    virtual void print(const xchar* string)
    {
        wcout << "super(" << _myName << "): " << string << endl;
    }

private:
    wstring _myName;
};

}

class Printer
{
public:
    Printer& usePrinter(Basic* printer)
    {
        wcout << "testing object as arg" << endl;
        printer->print(xSTRING("using printer"));
        return *this;
    }

    Basic& defPrinter()
    {
        wcout << "returning c++ -controlled object" << endl;
        return _myPrinter;
    }

private:
    Basic _myPrinter;
};

//
// bind code
//

#include <script/Declarator.h>

namespace script {

    SCRIPT_CLASS_REGISTER_BEGIN( Basic )
        .declareClass<Basic>(xSTRING("Basic"))
            .constructor< objOwn >(xSTRING("create"))
            .method(xSTRING("print"), &Basic::print)
            .method(xSTRING("ec"), (void (Basic::*)(int))&Basic::ec)
            .staticMethod(xSTRING("get10"), &Basic::get10)
// this is the same - because "plain" is default push-to-script policy:
//          .staticMethod<plain>(xSTRING("get10"), &Basic::get10)
    SCRIPT_CLASS_REGISTER_END( Basic )

    SCRIPT_CLASS_REGISTER_BEGIN( foo::Super )
        .declareNamespace(xSTRING("foo"))
            .declareClass<foo::Super, Basic>(xSTRING("Super"))
                .constructor< objOwn, const xchar* >(xSTRING("create"))
    SCRIPT_CLASS_REGISTER_END( foo::Super )

    SCRIPT_CLASS_REGISTER_BEGIN( Printer )
        .declareClass<Printer>(xSTRING("Printer"))
            .constructor< objOwn >(xSTRING("create"))
            .method< alreadyAsThis >(xSTRING("usePrinter"), &Printer::usePrinter)
            .method< objNoCare >(xSTRING("defPrinter"), &Printer::defPrinter)
    SCRIPT_CLASS_REGISTER_END( Printer )

}   //namespace script 

const xchar code_1[] = 
    xSTRING("\
            my10 <- Basic.get10();  \
            myBasic <- Basic.create();  \
            myBasic.print(\"testing argument \" + my10.tostring());    \
            mySuper1 <- foo.Super.create(\"first\");    \
            mySuper2 <- foo.Super.create(\"second\");   \
            mySuper1.print(\"testing inheritance\");   \
            mySuper2.print(\"testing inheritance again\");   \
            myPrinter <- Printer.create();  \
            myPrinter.usePrinter(myBasic);  \
            myPrinter.usePrinter(mySuper1).usePrinter(mySuper2); \
            myPrinter.usePrinter(myPrinter.myPrinter()); \
    ")
    ;

void testEvents(script::VMCore& core);
void testFields(script::VMCore& core);

int _tmain(int argc, _TCHAR* argv[])
{
    script::VMCore core(64);

    //actual binding to specific vm
    script::ClassTraits<Basic>::bind(&core);
    script::ClassTraits<foo::Super>::bind(&core);
    script::ClassTraits<Printer>::bind(&core);

    //using classes
    sq_compilebuffer(core.getVM(), code_1, wcslen(code_1), xSTRING("code_1"), false);
    sq_pushroottable(core.getVM());
    sq_call(core.getVM(), 1, false, true);

    //other tests
    testEvents(core);
    testFields(core);

	return 0;
}

