#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <stdio.h>
#include <tchar.h>
#include <iostream>
#include <string>
#include <string.h>

using namespace std;

#include <script/VMCore.h>
#include <script/Events.h>  //we wanna use events here

class Button
{
public:
    //declaration of the event. all events belongs to the instances of the classes
    script::Event<void,             //return type of the event - what c++ expects to get from script
        script::plain, int,         //first policy of the pushing to script, and first argument type
        script::plain, int          //second policy of the pushing to script, and first argument type
    >   onMousePressed;             //name of the event :))

    void mouse(int x, int y, bool pressed)
    {
        cout << "mouse function called with " << x << ", " << y << ", " << pressed << endl;
        if(pressed) {
            //checking if the event is assigned from script
            //calling of non-assigned event is prohibited!
            if(onMousePressed.isAssigned()) {
                cout << "calling script routine" << endl;
                //calling of event is VERY simple - just passing your declared argumens into "Call"
                onMousePressed.call(x, y);
            }

            //if event with return type (for examle, int), we whould use following:
            // 
            // script::Event<int,          - return type - int
            //      script::plain, int,    - first argument - int, passed with "plain" policy (best and the only policy for ints, floats, strings and other simple types)
            //      script::plain, int     - second argument - int, passed with "plain" policy
            // >   onMousePressed;       
            //
            // usage: int result = onMousePressed.call(x,y);
        }
    }

    static Button& defaultButton()
    {
        return _defaultButton;
    }

private:
    static Button _defaultButton;
};

Button Button::_defaultButton;

// binding declarations..
#include <script/Declarator.h>

namespace script {  //always use this namespace for such declarations. It is good practice.

    SCRIPT_CLASS_REGISTER_BEGIN( Button )
        .declareClass<Button>(xSTRING("Button"))
            //constructor - with no paramerters, -push-to-pscript policy - objOwn - means 
            // script will delete C++ Button instance, associated 
            // with script instance, when scipt instance will be erased
            .constructor< objOwn >(xSTRING("create"))

            //declaration of the event. Can u image it is so simple?
            .scriptEvent(xSTRING("onMousePressed"), &Button::onMousePressed)
            //
            // event declaration adds new function to the class with the name of the vent
            // this function returns embedded Event object to the scipt
            // Event object has following methods:
            //   setHandlerObject( callee, function )
            //      - assignns callee object and its method. 
            //   setHandler( function );    
            //      - assign a standallone function to event, calee is root table
            //   clear(); 
            //      - clears assignment
            //

            .method(xSTRING("mouse"), &Button::mouse)
            .staticMethod<objNoCare>(xSTRING("defaultButton"), &Button::defaultButton)
    SCRIPT_CLASS_REGISTER_END( Button )

}   //namespace script

const xchar events_code_1[] = 
    xSTRING("\
            defBtn <- Button.defaultButton();   \
            myBasic.print(\"assigning event\"); \
            defBtn.onMousePressed().setHandler( \
                function(x,y) {                 \
                    myBasic.print(x.tostring() + \" \" + y.tostring());            \
                }                \
            );    \
    ")
    ;

void testEvents(script::VMCore& core)
{
    //actual binding to specific vm
    script::ClassTraits<Button>::bind(&core);

    cout<<"-------------examle with events-----------"<<endl;

    //using classes
    sq_compilebuffer(core.getVM(), events_code_1, wcslen(events_code_1), xSTRING("events_code_1"), false);
    sq_pushroottable(core.getVM());
    sq_call(core.getVM(), 1, false, true);

    Button::defaultButton().mouse(10,20, false);
    Button::defaultButton().mouse(20,30, true);
    Button::defaultButton().mouse(33,45, true);
    Button::defaultButton().mouse(10,20, true);
    Button::defaultButton().mouse(4,2020, true);
}