#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <stdio.h>
#include <tchar.h>
#include <iostream>
#include <string>
#include <string.h>

using namespace std;

#include <script/VMCore.h>
#include <script/Events.h>  //we wanna use events here

class WithFields
{
public:
    WithFields() : x(0) {}

    int x;

    void printX()
    {
        cout << "C++ printX " << x << endl;
    }
};

// binding declarations..
#include <script/Declarator.h>

namespace script {  //always use this namespace for such declarations. It is good practice.

    SCRIPT_CLASS_REGISTER_BEGIN( WithFields )
        .declareClass<WithFields>(xSTRING("WithFields"))
            .constructor< objOwn >(xSTRING("create"))
            .method(xSTRING("printX"), &WithFields::printX)

            //
            // declaration of the field setter. Use it, if you want 
            // to let script change the value of the field directly
            // if you specify only setter, you'll get "write-only-variable"
            // for scripts
            //
            .setter(xSTRING("setX"), &WithFields::x)

            //
            // declaration of the field getter. Use it, if you want to
            // let scripts reading of the field directly. If you specify
            // only getter, you'll get "read-only-variable"
            //
            .getter(xSTRING("getX"), &WithFields::x)
    
            //
            // this declaration is the same as above - plain is default
            // push-to-script policy
            //
//            .getter<plain>(xSTRING("getX"), &WithFields::x)

    SCRIPT_CLASS_REGISTER_END( WithFields )

}   //namespace script

const xchar fields_code_1[] = 
    xSTRING("\
            wf <- WithFields.create();   \
            myBasic.print(\"getting fields value - \"); \
            myBasic.print(wf.getX().tostring()); \
            myBasic.print(\"setting fields value - \"); \
            wf.setX(112233);    \
            wf.printX();        \
            myBasic.print(\"getting fields value - \"); \
            myBasic.print(wf.getX().tostring()); \
    ")
    ;

void testFields(script::VMCore& core)
{
    //actual binding to specific vm
    script::ClassTraits<WithFields>::bind(&core);

    cout<<"-------------examle with fields-----------"<<endl;

    //using classes
    sq_compilebuffer(core.getVM(), fields_code_1, wcslen(fields_code_1), xSTRING("fields_code_1"), false);
    sq_pushroottable(core.getVM());
    sq_call(core.getVM(), 1, false, true);
}